/****************************************************************

  drill.c  -  vocabulary drill program                     

-----------------------------------------------------------
  created:  11/28/94

 DISTRIBUTION NOTICE

Copyright (c) 1994 James Kuffner, Jr. All rights reserved.

Permission to use, copy, modify and distribute this software for any
purpose is hereby granted without fee, provided that the above
copyright notice and this permission notice appear in all copies of
this software and that you do not sell the software.

THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY OF ANY KIND,
EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.

I developed the program in a UNIX environment, but ported it to the
Macintosh platform using Think C.  I apologize for the crummy user
interface.  If I ever have the time, I will try to improve it.  As
stated above, this software is free, but I enjoy comments and
suggestions.

Author:
   James Kuffner, Jr
   Computer Science Department
   Stanford University
   U.S.A.
   email: kuffner@stanford.edu
   http://robotics.stanford.edu/~kuffner

****************************************************************/

#include <stdio.h>
#include <string.h>

#define MAX_WORDS 400
#define MAX_LEN   30

#define CMD_DRILL 1
#define CMD_RVW   2
#define CMD_PRINT 3
#define CMD_FILE  4
#define CMD_QUIT  5
#define EMPTY     -1
#define CORRECT   0

/* MAC USER INTERFACE VARIABLES */
#define ErrorAlert	256
static Point SFGwhere = { 30, 82 };
static Point SFPwhere = { 46, 104 };
static SFReply reply;


FILE *fpin;
char *wordFileName = "wordlist.txt";
char word[MAX_WORDS][MAX_LEN];
char def[MAX_WORDS][MAX_LEN];
int used[MAX_WORDS];
int wrong[MAX_WORDS];
int numwords, correct;


/**************************/
void ReadAnswer(answer)
     char *answer;
{
  char c;
  int i;
  
  i = 0;
  fflush(stdin);
  while((answer[i++] = getchar()) != '\n');
  answer[i-1] = '\0';
}

/**************************/
void Drill()
{
  char	answer[32];
  int		len, i, j;
  long	r;
  
  printf("Randomizing words...");
  for (i=0; i<numwords; i++) {
    used[i] = EMPTY;
    wrong[i] = FALSE;
  }
  for (i=0; i<numwords; i++) {
    while (used[r = Random() % numwords] != EMPTY);
    used[r] = i;
  }
  correct = 0;
  for (i=0; i<numwords; i++) {
    printf("\n%d> English: %-30s  Japanese: ", i+1, def[used[i]]);
    ReadAnswer(answer);
    if (strcmp(answer, word[used[i]]) == CORRECT) {
      correct++;
      printf("\n\tCorrect!\n");
    }
    else {
      wrong[used[i]] = TRUE;
      printf("\n\tWrong! The correct answer is: %s\n", word[used[i]]);
    }
  }
  printf("\n\nYou got %d out of %d words correct.\n\n", correct, numwords);
}

/**************************/
void Review()
{
  char answer[32];
  int  len, i, c, incorrect;
  long r;
  
  printf("\nREVIEW OF MISSED WORDS\n");
  incorrect = numwords - correct;
  printf("Randomizing words...\n");
  for (i=0; i<numwords; i++) {
    used[i] = EMPTY;
  }
  for (i=0; i<numwords; i++) {
    if (wrong[i] == TRUE) {
      while (used[r = Random() % incorrect] != EMPTY);
      used[r] = i;
    }
  }
  c = 0;
  for (i=0; i<incorrect; i++) {
    printf("\n%d> English: %-30s  Japanese: ", i+1, def[used[i]]);
    ReadAnswer(answer);
    if (strcmp(answer, word[used[i]]) == CORRECT) {
      correct++;
      c++;
      wrong[used[i]] = FALSE;
      printf("\n\tCorrect!\n");
    }
    else {
      printf("\n\tWrong! The correct answer is: %s\n", word[used[i]]);
    }
  }
  printf("\nYou got %d out of %d review words correct.\n", c, incorrect);
  printf("This gives you a total score of %d out of %d.\n\n", correct, numwords);
}

/**************************/
int pStrCopy (StringPtr p1, StringPtr p2)
     
     /* copies a pascal string from p1 to p2 */
{
  register int len;
  
  len = *p2++ = *p1++;
  while (--len>=0) *p2++=*p1++;
}

/**************************/
int pToCStr (StringPtr p1, char *p2)
     
     /* copies a pascal string p1 to a C string p2 */
{
  register int len;
  
  len = *p1++;
  while (--len>=0) *p2++=*p1++;
  *p2 = '\0';
}

/**************************/
int OldFile (Str255 fn, int *vRef) {
  SFTypeList	myTypes;
  
  myTypes[0]='TEXT';
  
  SFGetFile(SFGwhere, "\p", 0L, 1, myTypes, 0L, &reply );
  
  if (!reply.good)
    return (0);
	else {
	  pStrCopy(reply.fName, fn);
	  *vRef = reply.vRefNum;
	  return(1);
	}
}

/**************************/
int FileError(Str255 s, Str255 f)
     
{
  ParamText(s, f,"\p", "\p");
  Alert(ErrorAlert, 0L);
}

/**************************/
void InputWords()
{
  char  s[80], temp[80];
  char *curr;
  int 	vRef, refNum, io, Success, errnum;
  Str255	fn;
  char  fileName[20];
  
  Success = FALSE;
  errnum = 0;
  while (Success != TRUE) {
    printf("Select vocabulary file...\n");
    printf("<NOTE: the vocabulary file must be in the same\n");
    printf("       folder as the application program>\n");
    if (OldFile(fn, &vRef ))
      pToCStr(fn, fileName);
    else
      sprintf(fileName, "%s", wordFileName);
    
    fpin = fopen(fileName, "r");	
    if (((void *) fpin) == NULL) {
      FileError("\pError opening ", "\pVocabulary File");
      if (errnum++ > 2) {
	printf("\n  File input error!  Move a valid vocabulary file into");
	printf("\n  the same folder as this application and try again.");
	exit(-1);
      }
    }
    else
      Success = TRUE;
  }
  numwords = 0;
  printf("\n\nReading words from file %s ...", fileName);
  
  while ((fscanf(fpin, "%s", s) != EOF) && (numwords < MAX_WORDS)) {
    strcpy(temp,"");
    while (strcmp(s,"#") != 0) {
      sprintf(temp,"%s %s",temp,s);
      fscanf(fpin, "%s", s);
    }
    if (strlen(temp) < MAX_LEN)
      strcpy(word[numwords], temp+1);
    else
      printf("ERROR - word too long: %s\n", temp);
    fscanf(fpin, "%s", s);
    strcpy(temp,"");
    while (strcmp(s,"#") != 0) {
      sprintf(temp,"%s %s",temp,s);
      fscanf(fpin, "%s", s);
    }
    if (strlen(temp) < MAX_LEN)
      strcpy(def[numwords], temp+1);
    else
      printf("ERROR - word too long: %s\n", temp);
    numwords++;
  }
  printf(" %d words read.\n\n\n", numwords);
  fclose(fpin);
}

/**************************/
void PrintWords()
{
  int i;
  
  printf("\nJAPANESE                      ENGLISH");
  printf("\n----------------------------- -----------------------------\n");
  for (i=0; i<numwords; i++) {
    printf("%-29s %-29s\n",word[i],def[i]);
    if ((i % 20) == 19) {
      fflush(stdin);
      printf("    <press return to continue>");
      getchar();        /* pause during printing for "more" effect */
    }
  }    
  printf("\n\n");
}


/**************************/
void PrintIntro() {
  printf("\n\t\tJAPANESE VOCABULARY DRILL\n");
  printf("\t\t-------------------------\n");
  printf("\t\t by: James Kuffner, Jr.\n");
  printf("\t\t  Stanford University\n");
  printf("\t\t   November 28, 1994\n\n");
}

/**************************/
int GetCmd() {
  char c='0';
  char st[10];

  fflush(stdin);
  while((c < '1')||(c >'5')) {
    printf("Please select one of the following:\n");
    printf("\t1) English->Japanese drill\n");
    printf("\t2) Review words that you missed\n");  
    printf("\t3) Print all words currently loaded\n"); 
    printf("\t4) Load a new vocabulary list\n");
    printf("\t5) Quit application\n\n");
    printf("Your Choice (1-5):");
    scanf("%s", st);
    if (strlen(st) >0)
      c = st[0];
  }
  return((int) (c - '0'));
}

/**************************/
void PromptUser()
{
  int cmd, newlist;
  
  newlist = TRUE;
  cmd = CMD_DRILL;
  while (cmd != CMD_QUIT) {
    cmd = GetCmd();
    if (cmd == CMD_DRILL) {
      printf("\nENGLISH TO JAPANESE DRILL\n\n");
      Drill();
      newlist = FALSE;
    }	
    else if (cmd == CMD_RVW) {
      if (newlist != TRUE)
	if (correct == numwords)
	  printf("\nYou got all of the words correct (you stud!)\n\n");
	else
	  Review();
      else
	printf("\n\tYou must go through the drill before you can review!\n\n");
    }	
    else if (cmd == CMD_PRINT) {
      PrintWords();
    }
    else if (cmd == CMD_FILE) {
      InputWords();
      newlist = TRUE;
    }
  }
  printf("\nSayoonara!   <press 'return' to exit>\n");
}

/**************************/
int main()
{
  PrintIntro();
  InputWords();
  PromptUser();
}
